' 32feet.NET - Personal Area Networking for .NET
'
' InTheHand.Net.Bluetooth.BluetoothRadio
' 
' Copyright (c) 2007 In The Hand Ltd, All rights reserved.
' Copyright (c) 2007 Alan J McFarlane.
' This source code is licensed under the In The Hand Community License - see License.txt

Imports InTheHand.Net
Imports InTheHand.Net.Sockets
Imports InTheHand.Net.Bluetooth
Imports InTheHand.Net.Bluetooth.AttributeIds
Imports System.Net.Sockets

Public Class Form1
    '===================================================================
    Shared ReadOnly NewLine As String = Environment.NewLine

    Function MessageBox_Show(ByVal owner As Form, ByVal text As String, ByVal caption As String, _
    ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon, ByVal defaultButton As MessageBoxDefaultButton) _
    As System.Windows.Forms.DialogResult
        Return MessageBox.Show(owner, text, caption, buttons, icon, defaultButton)
    End Function

    Sub MessageBox_Show(ByVal owner As Form, ByVal text As String, ByVal caption As String)
        MessageBox.Show(owner, text, caption)
    End Sub

    Sub MessageBox_Show(ByVal owner As Form, ByVal text As String)
        MessageBox.Show(owner, text)
    End Sub

    '===================================================================
    Private m_devices() As BluetoothDeviceInfo
    Private m_startTime, m_endTime As DateTime
    Private m_lastRecords()() As Byte
    Private m_lastRecordsParsed() As ServiceRecord
    Private m_lock As New Object 'To apply 'volatile' access to the m_devices shared field.
    Const m_noBluetooth As Boolean = False

    '--------------------
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Fill the list of devices with one for the local device (fake from radio info)
        Dim devices() As BluetoothDeviceInfo = {} ' Want a zero length array
        FillDevicesFill(devices)
        '----
        Form1Connect_Load(sender, e)
        Form1Server_Load(sender, e)
        '
        TabControl1_SelectedIndexChanged(Nothing, Nothing)
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        Form1Connect_FormClosing()
    End Sub

    '===========================================================================
    Private Sub MenuItemNewBtCli_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemNewBtCli.Click
        Try
            Dim foo As New BluetoothClient
        Catch sex As SocketException
            MessageBox_Show(Me, sex.NativeErrorCode & " / " & sex.Message, "New BluetoothClient")
        End Try
    End Sub

    Private Sub MenuItemBtCliDiscover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemBtCliDiscover.Click
        Dim cli As BluetoothClient
        Try
            cli = New BluetoothClient
        Catch sex As SocketException
            MessageBox_Show(Me, sex.NativeErrorCode & " / " & sex.Message, "New BluetoothClient")
            Return
        End Try
        Try
            Dim arr() As BluetoothDeviceInfo = cli.DiscoverDevices()
            MessageBox_Show(Me, "Discovered " & arr.Length.ToString() & " devices.")
        Catch sex As SocketException
            MessageBox_Show(Me, sex.NativeErrorCode & " / " & sex.Message, "New BluetoothClient")
        End Try
    End Sub

    Private Sub MenuItemSetRadioName_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemSetRadioName.Click
        Dim radio As BluetoothRadio = BluetoothRadio.PrimaryRadio
        Dim newName As String = InputBox("Passcode/PIN", "Set Radio Name", radio.Name)
        If newName.Length = 0 Then
            MessageBox_Show(Me, "Cancelled", "Set Radio Name")
            Return
        End If
        radio.Name = newName
    End Sub

    Private Sub MenuItemSecurityRemoveDevice_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemSecurityRemoveDevice.Click
        Dim bdi As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If bdi Is Nothing Then
            MessageBox_Show(Me, "No device selected", "Remove Device")
        Else
            Dim rslt As DialogResult _
                = MessageBox_Show(Me, "Device: " & NewLine & bdi.DeviceName & NewLine & bdi.DeviceAddress.ToString(), _
                    "Remove Device", _
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2)
            If rslt = Windows.Forms.DialogResult.Yes Then
                Dim result As Boolean = BluetoothSecurity.RemoveDevice(bdi.DeviceAddress)
                MessageBox_Show(Me, "Result: " & result, "Remove Device")
            End If
        End If
    End Sub

    Private Sub MenuItemSecurityPairRequest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemSecurityPairRequest.Click
        Dim bdi As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If bdi Is Nothing Then
            MessageBox_Show(Me, "No device selected", "Pair Request")
        Else
            Dim rslt As DialogResult _
                = MessageBox_Show(Me, "Device: " & NewLine & bdi.DeviceName & NewLine & bdi.DeviceAddress.ToString(), _
                    "Pair Request", _
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2)
            If rslt = Windows.Forms.DialogResult.Yes Then
                Dim passcode As String = InputBox("Passcode/PIN", "Pair Request", "0000")
                If passcode.Length = 0 Then
                    MessageBox_Show(Me, "Cancelled", "Pair Request")
                    Return
                End If
                Dim result As Boolean = BluetoothSecurity.PairRequest(bdi.DeviceAddress, passcode)
                MessageBox_Show(Me, "Result: " & result, "Pair Request")
            End If
        End If
    End Sub

    '===========================================================================
#Region "Discovery"
    Private Sub ButtonDiscoverDevices_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonDiscoverDevices.Click, ButtonClientDiscoverDevices.Click, MenuItemDiscoverAll.Click
        Dim newOnly As Boolean = False
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf BackgroundDiscoverAndFill, newOnly)
    End Sub

    Private Sub ButtonDiscoverDevicesNewOnly_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDiscoverNewOnly.Click
        Dim newOnly As Boolean = True
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf BackgroundDiscoverAndFill, newOnly)
    End Sub

    Private Sub ButtonDiscoverDevicesAuthenticatedOnly_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDiscoverAuthenticatedOnly.Click
        Dim flags As New DiscoveryFlags(True, False, False)
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf BackgroundDiscoverAndFill3, flags)
    End Sub

    Private Sub ButtonDiscoverDevicesRememberedOnly_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDiscoverRememberedOnly.Click
        Dim flags As New DiscoveryFlags(False, True, False)
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf BackgroundDiscoverAndFill3, flags)
    End Sub

    Structure DiscoveryFlags
        Friend ReadOnly m_authenticated, m_remembered, m_unknown As Boolean

        Sub New(ByVal authenticated As Boolean, ByVal remembered As Boolean, ByVal unknown As Boolean)
            m_authenticated = authenticated
            m_remembered = remembered
            m_unknown = unknown
        End Sub
    End Structure

    Private Sub BackgroundDiscoverAndFill(ByVal dummy As Object)
        Dim newOnly As Boolean = CBool(dummy)
        Dim flags As New DiscoveryFlags(Not newOnly, Not newOnly, True)
        BackgroundDiscoverAndFill3(flags)
    End Sub

    Private Sub BackgroundDiscoverAndFill3(ByVal dummy As Object)
        Dim flags As DiscoveryFlags = CType(dummy, DiscoveryFlags)
        If m_noBluetooth Then
            Beep()
            Return
        End If
        Dim cli As New BluetoothClient()
        Dim startTime As DateTime = DateTime.UtcNow
        Dim devices() As BluetoothDeviceInfo = cli.DiscoverDevices(255, flags.m_authenticated, _
            flags.m_remembered, flags.m_unknown)
        Dim endTime As DateTime = DateTime.UtcNow
        SyncLock m_lock
            m_startTime = startTime
            m_endTime = endTime
            m_devices = devices
        End SyncLock
        Dim handler As EventHandler = AddressOf FillDevices
        Me.BeginInvoke(handler)
    End Sub

    Private Sub FillDevices(ByVal sender As Object, ByVal e As EventArgs)
        System.Diagnostics.Debug.Assert(Not Me.InvokeRequired)
        Dim devices() As BluetoothDeviceInfo = Nothing
        Dim startTime, endTime As DateTime
        ' Get the device list set by the background thread.
        SyncLock m_lock
            devices = m_devices
            startTime = m_startTime
            endTime = m_endTime
        End SyncLock
        System.Diagnostics.Debug.Assert(Not devices Is Nothing)
        '
        FillDevicesFill(devices)
        DumpDeviceInfo(devices, startTime, endTime)
    End Sub

    Private Sub FillDevicesFill(ByVal devices() As BluetoothDeviceInfo)
        ' Add a special device entry for the local device.
        devices = AppendLocalDeviceInfo(devices)
        '
        FillDevicesFillNoAddLocal(devices)
    End Sub

    Private Sub FillDevicesFillNoAddLocal(ByVal devices() As BluetoothDeviceInfo)
        ' Apply to the combobox via data binding
        Me.BluetoothDeviceInfoBindingSource.SuspendBinding()
        Me.BluetoothDeviceInfoBindingSource.Clear()
        For Each item As BluetoothDeviceInfo In devices
            Me.BluetoothDeviceInfoBindingSource.Add(item)
        Next
        Me.BluetoothDeviceInfoBindingSource.ResumeBinding()
        Beep()
    End Sub

    ' Adds a special device entry for the local device.
    Private Function AppendLocalDeviceInfo(ByVal devices() As BluetoothDeviceInfo) As BluetoothDeviceInfo()
        Dim localRadio As BluetoothRadio = BluetoothRadio.PrimaryRadio
        If localRadio Is Nothing Then
            Dim rslt As System.Windows.Forms.DialogResult = MessageBox_Show(Me, _
                "There is no Bluetooth hardware, or it uses unsupported software. Quit?", _
                "No Bluetooth support", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1)
            If rslt <> Windows.Forms.DialogResult.No Then
                Application.Exit()
            End If
        End If
        System.Diagnostics.Debug.Assert(Not (localRadio Is Nothing And devices.Length <> 0), "PrimaryRadio is null -- but DiscoverDevices worked!")
        If Not localRadio Is Nothing Then ' Handle gracefully anyway!
            ' Get the local MAC Address and create a device object.
            Dim localAddr As BluetoothAddress = localRadio.LocalAddress
            Dim localBdi As New BluetoothDeviceInfo(localAddr)
            localBdi.DeviceName = "- local device -"
            ' Copy the entries into a new array with the 'local' item at the front.
            Dim devicesPlusLocal(devices.Length) As BluetoothDeviceInfo
            devices.CopyTo(devicesPlusLocal, 1)
            devicesPlusLocal(0) = localBdi
            devices = devicesPlusLocal
        End If
        Return devices
    End Function

    Private Sub DumpDeviceInfo(ByVal devices() As BluetoothDeviceInfo, _
            ByVal startTime As DateTime, ByVal endTime As DateTime)
        Dim sb As New System.Text.StringBuilder
        Me.TextBox1.Text = String.Empty
        Dim localTime As DateTime = startTime.ToLocalTime
        sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
            "Discovery process started at {0} UTC, {1} local, and ended at {2} UTC.", _
            startTime, localTime, endTime)
        sb.Append(vbCrLf + vbCrLf)
        Me.TextBox1.Text += sb.ToString()
        For Each curDevice As BluetoothDeviceInfo In devices
            sb.Length = 0
            sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                "* {0}", curDevice.DeviceName)
            sb.Append(vbCrLf)
            sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                "Address: {0}", curDevice.DeviceAddress)
            sb.Append(vbCrLf)
            sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                "Remembered: {2}, Authenticated: {0}, Connected: {1}", _
                curDevice.Authenticated, curDevice.Connected, curDevice.Remembered)
            sb.Append(vbCrLf)
            sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                "LastSeen: {0}, LastUsed: {1}", _
                curDevice.LastSeen, curDevice.LastUsed)
            sb.Append(vbCrLf)
            DumpCodInfo(curDevice.ClassOfDevice, sb)
            sb.Append(vbCrLf)
            Me.TextBox1.Text += sb.ToString()
        Next
    End Sub

    Shared Sub DumpCodInfo(ByVal cod As ClassOfDevice, ByVal sb As System.Text.StringBuilder)
        sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
            "CoD: (0x{0:X6})", cod.Value, cod)
        sb.Append(vbCrLf)
        sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
            " Device:  {0} (0x{1:X2}) / {2} (0x{3:X4})", cod.MajorDevice, CInt(cod.MajorDevice), cod.Device, CInt(cod.Device))
        sb.Append(vbCrLf)
        sb.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
            " Service: {0} (0x{1:X2})", cod.Service, CInt(cod.Service))
        sb.Append(vbCrLf)
    End Sub

    Private Function GetSelectedDeviceInfo() As BluetoothDeviceInfo
        Dim selectedDevice As BluetoothDeviceInfo = DirectCast(Me.ComboBoxDevices.SelectedItem, BluetoothDeviceInfo)
        If selectedDevice Is Nothing Then
            MessageBox_Show(Me, "No Device selected.")
        End If
        Return selectedDevice
    End Function

#End Region 'Discovery

    '===========================================================================
#Region "SDP"
    '----------------
    Private Sub ButtonGetOppRecord_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonGetOppRecord.Click
        Dim searchUuid As Guid = BluetoothService.ObexObjectPush
        DisplayAllRecordsWithUuid(searchUuid, GetType(ObexAttributeId))
    End Sub

    Private Sub ButtonGetInstalledServices_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonGetInstalledServices.Click
        Dim bdi As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If bdi Is Nothing Then Exit Sub
        Dim svcUuids() As Guid = bdi.InstalledServices
        If svcUuids.Length = 0 Then
            Me.TextBox1.Text = "InstalledServices returned no services."
            Exit Sub
        End If
        Me.TextBox1.Text = String.Empty
        ReDim Me.m_lastRecords(svcUuids.Length - 1)
        ReDim Me.m_lastRecordsParsed(svcUuids.Length - 1)
        Dim i As Integer
        For Each curUuid As Guid In svcUuids
            Dim text As String
            Try
                Dim rawRecords()() As Byte = bdi.GetServiceRecordsUnparsed(curUuid)
                If rawRecords.Length = 0 Then
                    text = "No records."
                Else
                    System.Diagnostics.Debug.Assert(rawRecords.Length = 1)
                    m_lastRecords(i) = rawRecords(0)
                    Dim recordBytes() As Byte = rawRecords(0)
                    '
                    Dim parser As New ServiceRecordParser 'TODO ButtonGetInstalledServices_Click parser!!!
                    Dim record As ServiceRecord = parser.Parse(recordBytes)
                    m_lastRecordsParsed(i) = record
                    text = ServiceRecordUtilities.Dump(record)
                End If
            Catch sex As System.Net.Sockets.SocketException
                text = String.Format(System.Globalization.CultureInfo.InvariantCulture, _
                    "Failed to read ServiceRecord with: {0} (0x{0:X}) '{1}'", _
                    sex.ErrorCode, sex.Message)
            End Try
            Me.TextBox1.Text += ChrW(&H2022) + " Record with UUID: " + curUuid.ToString() + NewLine + NewLine
            Me.TextBox1.Text += text
            Me.TextBox1.Text += NewLine + NewLine
            i += 1
        Next
    End Sub

    Private Sub ButtonAllOverL2cap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAllOverL2cap.Click, MenuItemQueryAllL2cap.Click
        Dim searchUuid As Guid = BluetoothService.L2CapProtocol
        DisplayAllRecordsWithUuid(searchUuid)
    End Sub

    Private Sub MenuItemQuerySelectedClass_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemQuerySelectedClass.Click
        Dim searchUuid As Guid = GetSelectedService()
        If Guid.Empty.Equals(searchUuid) Then
            MessageBox_Show(Me, "Invalid Service UUID.")
            Return
        End If
        DisplayAllRecordsWithUuid(searchUuid)
    End Sub

    '----------------

    Private Sub DisplayAllRecordsWithUuid(ByVal searchUuid As Guid, ByVal ParamArray attributeIdDefiningClasses() As Type)
        Dim bdi As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If bdi Is Nothing Then Exit Sub
        Dim rawRecords()() As Byte
        Dim startTicks, elapsedQuery, elapsedDisplay As Integer
        Try
            startTicks = Environment.TickCount
            rawRecords = bdi.GetServiceRecordsUnparsed(searchUuid)
            elapsedQuery = Environment.TickCount - startTicks
            Me.TextBox1.Text = "Query complete"
        Catch sex As System.Net.Sockets.SocketException
            Me.TextBox1.Text = String.Format(System.Globalization.CultureInfo.InvariantCulture, _
                "Failed to read ServiceRecord with: {0} (0x{0:X}) '{1}'", _
                sex.SocketErrorCode, sex.Message)
            Exit Sub
        End Try
        startTicks = Environment.TickCount
        DisplayRecords(rawRecords, attributeIdDefiningClasses)
        elapsedDisplay = Environment.TickCount - startTicks
        Me.TextBox1.Text += String.Format(System.Globalization.CultureInfo.InvariantCulture, _
            "Elapsed (ms): Query: {0}, Display: {1}" + NewLine + NewLine, _
            elapsedQuery, elapsedDisplay)
    End Sub

    Private Sub DisplayRecords(ByVal rawRecords()() As Byte, ByVal ParamArray attributeIdDefiningClasses() As Type)
        If rawRecords.Length = 0 Then
            Me.TextBox1.Text = "Search returned zero services." + NewLine + NewLine
            Exit Sub
        End If
        Me.TextBox1.Text = String.Empty
        Me.m_lastRecords = rawRecords
        System.Diagnostics.Debug.Assert(Me.m_lastRecords.Length >= 1)
        ReDim Me.m_lastRecordsParsed(Me.m_lastRecords.Length - 1)
        System.Diagnostics.Debug.Assert(Me.m_lastRecords.Length = Me.m_lastRecordsParsed.Length)
        '
        Dim i As Integer = 0
        Dim parser As New ServiceRecordParser
        parser.SkipUnhandledElementTypes = True ' display what we can of the record
        For Each rawRecord As Byte() In rawRecords
            Me.TextBox1.Text += ChrW(&H2022) + " Record:" + NewLine + NewLine
            Dim recordBytes() As Byte = rawRecord
            '
            Try
                Dim record As ServiceRecord = parser.Parse(recordBytes)
                Me.m_lastRecordsParsed(i) = record
                Dim text As String = ServiceRecordUtilities.Dump(record, attributeIdDefiningClasses)
                Me.TextBox1.Text += text
            Catch ex As Exception
                Me.TextBox1.Text += "Parse/Dump failed with: " & ex.ToString
            End Try
            Me.TextBox1.Text += NewLine + NewLine
            i += 1
        Next
    End Sub

    Private Sub ButtonBytesOfLastRecords_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonBytesOfLastRecords.Click, MenuItemDumpRawRecordsToolStripMenuItem.Click
        If m_lastRecords Is Nothing Then Exit Sub
        'Append, or clear before appending...? Me.TextBox1.Text = String.Empty
        For Each curRecord As Byte() In m_lastRecords
            Dim bytes() As Byte = curRecord
            Dim text As String = MakeArrayContent(bytes)
            Me.TextBox1.Text += text
            Me.TextBox1.Text += NewLine + NewLine
        Next
    End Sub

    Private Shared Function MakeArrayContent(ByVal buffer() As Byte) As String
        Dim bldr As New System.Text.StringBuilder
        Dim i As Integer
        For Each cur As Byte In buffer
            bldr.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, "0x{0:x2}, ", cur)
            '
            i += 1
            If i Mod 16 = 0 Then
                bldr.Append(NewLine)
            ElseIf i Mod 8 = 0 Then
                bldr.Append(" ")
            End If
        Next
        Return bldr.ToString()
    End Function

    '---------------------------------------------------------------------------
    Public Function GetServiceName(ByVal record As ServiceRecord) As String
        Dim primaryLang As LanguageBaseItem = record.GetPrimaryLanguageBaseItem()
        If primaryLang Is Nothing Then
            ' Primary multi-language not set, would have to guess strings' encoding,
            ' what LanguageBase would have been added to it.
            ' Lets assume the 0x0100 value.
            Dim attrId As ServiceAttributeId = ServiceRecord.CreateLanguageBasedAttributeId( _
                UniversalAttributeId.ServiceName, LanguageBaseItem.PrimaryLanguageBaseAttributeId)
            If record.Contains(attrId) Then
                Dim attr As ServiceAttribute = record.GetAttributeById(attrId)
                Dim element As ServiceElement = attr.Value
                If element.ElementType = ElementType.TextString Then
                    ' Lets guess UTF-8/ASCII/Windows-1252 ASCII range, and not UTF-16 etc
                    Dim sn As String = element.GetValueAsStringUtf8()
                    Console.WriteLine("ServiceName (guessed encoding): " & sn)
                    Return sn
                End If
            End If
        ElseIf record.Contains(UniversalAttributeId.ServiceName, primaryLang) Then
            Dim sn As String = record.GetMultiLanguageStringAttributeById(UniversalAttributeId.ServiceName, primaryLang)
            Console.WriteLine("ServiceName: " & sn)
            Return sn
        End If
        ' Failed, return null
        Return Nothing
    End Function

    Function GetRfcommChannelNumber(ByVal record As ServiceRecord) As Byte
        Const none As Byte = 0
        Dim attr As ServiceAttribute = record.GetAttributeById(UniversalAttributeId.ProtocolDescriptorList)
        Dim e0 As ServiceElement = attr.Value
        If e0.ElementType = ElementType.ElementAlternative Then
            Console.WriteLine("Don't support alternative ProtocolDescriptorList values.")
            Return none
        ElseIf e0.ElementType <> ElementType.ElementSequence Then
            Console.WriteLine("Bad ProtocolDescriptorList content.")
            Return none
        End If
        Dim protoStack As IList(Of ServiceElement) = e0.GetValueAsElementList()
        Dim etor As IEnumerator(Of ServiceElement) = protoStack.GetEnumerator()
        Dim layer As ServiceElement
        Dim layerContent As IList(Of ServiceElement)
        If Not etor.MoveNext() Then
            Console.WriteLine("Protocol stack truncated before {0}.", "L2CAP")
            Return none
        End If
        layer = etor.Current
        layerContent = layer.GetValueAsElementList()
        If layerContent(0).GetValueAsUuid() <> BluetoothService.L2CapProtocol Then
            Console.WriteLine("Bad protocol stack, layer {0} is not {1}.", 1, "L2CAP")
            Return none
        End If
        If Not etor.MoveNext() Then
            Console.WriteLine("Protocol stack truncated before {0}.", "RFCOMM")
            Return none
        End If
        layer = etor.Current
        layerContent = layer.GetValueAsElementList()
        If layerContent(0).GetValueAsUuid() <> BluetoothService.RFCommProtocol Then
            Console.WriteLine("Bad protocol stack, layer {0} is not {1}.", 2, "RFCOMM")
            Return none
        End If
        Dim channelNumber As Byte = CByte(layerContent(1).Value)
        Console.WriteLine("Channel Number: {0}.", channelNumber)
        Return channelNumber
    End Function

    Public Function GetServiceClasses(ByVal record As ServiceRecord) As String
        Dim bldr As New System.Text.StringBuilder
        If Not record.Contains(UniversalAttributeId.ServiceClassIdList) Then
            Return "no class list!"
        End If
        Dim attr As ServiceAttribute = record.GetAttributeById(UniversalAttributeId.ServiceClassIdList)
        Dim seq As ServiceElement = attr.Value
        If Not seq.ElementType = ElementType.ElementSequence Then
            Return "attribute not a seq!"
        End If
        Const Sep As String = ", "
        For Each e As ServiceElement In seq.GetValueAsElementList()
            Dim name As String = Nothing
            If e.ElementType = ElementType.Uuid16 Then
                name = BluetoothService.GetName(CType(e.Value, UInt16))
                bldr.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                    "0x{0:X4}", e.Value)
            ElseIf e.ElementType = ElementType.Uuid32 Then
                name = BluetoothService.GetName(CType(e.Value, UInt32))
                bldr.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                    "0x{0:X8}", e.Value)
            ElseIf e.ElementType = ElementType.Uuid128 Then
                name = BluetoothService.GetName(e.GetValueAsUuid)
                bldr.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                    "{0}", e.GetValueAsUuid)
            End If
            If Not name Is Nothing Then
                bldr.AppendFormat(System.Globalization.CultureInfo.InvariantCulture, _
                    " {0}", name)
            End If
            bldr.Append(Sep)
        Next
        bldr.Length -= Sep.Length
        Return bldr.ToString()
    End Function

    Private Sub ButtonNameAndChannelOfLastRecords_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonNameAndChannelOfLastRecords.Click, MenuItemDumpRecordsNameChannelToolStripMenuItem.Click
        If m_lastRecordsParsed Is Nothing Then Exit Sub
        Me.TextBox1.Text += "--------------------------------" & NewLine
        For Each record As ServiceRecord In m_lastRecordsParsed
            If record Is Nothing Then
                ' Parse of that record failed...
                Me.TextBox1.Text += "N/A" + NewLine
            Else
            Dim name As String = GetServiceName(record)
            Dim classes As String = GetServiceClasses(record)
            Dim channelNumber As Byte = GetRfcommChannelNumber(record)
            '
            If name Is Nothing Then
                Me.TextBox1.Text += "ServiceName: " & "<none>" & NewLine
            Else
                Me.TextBox1.Text += "ServiceName: '" & name & "'" & NewLine
            End If
            Me.TextBox1.Text += "ServiceClasses: " & classes & NewLine
            If channelNumber = 0 Then
                Me.TextBox1.Text += "Channel Num: " & "<not assigned>" & NewLine
            Else
                Me.TextBox1.Text += "Channel Num: " & channelNumber & NewLine
            End If
            End If
            Me.TextBox1.Text += NewLine
        Next
    End Sub

    Private Function CreateDummyDevices() As BluetoothDeviceInfo()
        Dim devices(2) As BluetoothDeviceInfo
        Const FakeOuiCiscoBaseUser As String = "02:00:0C:"
        devices(0) = New BluetoothDeviceInfo(BluetoothAddress.Parse(FakeOuiCiscoBaseUser & "00:00:00"))
        devices(1) = New BluetoothDeviceInfo(BluetoothAddress.Parse(FakeOuiCiscoBaseUser & "00:00:01"))
        devices(2) = New BluetoothDeviceInfo(BluetoothAddress.Parse(FakeOuiCiscoBaseUser & "00:00:02"))
        Return devices
    End Function

    Private Sub MenuItemAddTestDevices_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemAddTestDevices.Click
        Dim devices() As BluetoothDeviceInfo = CreateDummyDevices()
        FillDevicesFillNoAddLocal(devices)
    End Sub

    Private Sub MenuItemRadioInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemRadioInfo.Click
        Dim using1 As IDisposable = Nothing
        Try
            Dim wtr As New IO.StringWriter : using1 = wtr
            wtr.WriteLine("Primary radio:")
            DisplayPrimaryBluetoothRadio(wtr)
            wtr.WriteLine()
            wtr.WriteLine("All radios:")
            DisplayAllBluetoothRadios(wtr)
            Me.TextBox1.Text = wtr.ToString()
        Finally
            ' StringWriter doesn't use any native resources but 
            If Not using1 Is Nothing Then
                using1.Dispose()
            End If
        End Try
    End Sub

    Public Shared Sub DisplayPrimaryBluetoothRadio(ByVal wtr As System.IO.TextWriter)
        Dim myRadio As BluetoothRadio = BluetoothRadio.PrimaryRadio
        If myRadio Is Nothing Then
            wtr.WriteLine("No radio hardware or unsupported software stack")
            Return
        End If
        Dim mode As RadioMode = myRadio.Mode
        ' Warning: LocalAddress is null if the radio is powered-off.
        wtr.WriteLine("* Radio, address: {0:C}", myRadio.LocalAddress)
        wtr.WriteLine("Mode: " & mode.ToString())
        wtr.WriteLine("Name: " & myRadio.Name _
            & ", LmpSubversion: " & myRadio.LmpSubversion)
        wtr.WriteLine("ClassOfDevice: " & myRadio.ClassOfDevice.ToString() _
            & ", device: " & myRadio.ClassOfDevice.Device.ToString() _
            & " / service: " & myRadio.ClassOfDevice.Service.ToString())
        '
        ' Enable discoverable mode
        'wtr.WriteLine()
        'myRadio.Mode = RadioMode.Discoverable
        'wtr.WriteLine("Radio Mode now: " & myRadio.Mode.ToString())
    End Sub

    Public Shared Sub DisplayAllBluetoothRadios(ByVal wtr As System.IO.TextWriter)
        Dim radios() As BluetoothRadio = BluetoothRadio.AllRadios
        If radios.Length = 0 Then
            wtr.WriteLine("No radio hardware or unsupported software stack")
            Return
        End If
        Dim tmpSb As New System.Text.StringBuilder
        For Each curRadio As BluetoothRadio In radios
            wtr.WriteLine("* Radio, address: {0:C}", curRadio.LocalAddress)
            wtr.WriteLine("Mode: " & curRadio.Mode.ToString())
            wtr.WriteLine("Name: " & curRadio.Name _
                & ", LmpSubversion: " & curRadio.LmpSubversion)
            tmpSb.Length = 0
            DumpCodInfo(curRadio.ClassOfDevice, tmpSb)
            wtr.Write(tmpSb.ToString)
            wtr.WriteLine()
        Next
    End Sub

    '==
    Private Sub TabControl1_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TabControl1.SelectedIndexChanged
        Const xx As Integer = 0
        If Me.TabControl1.SelectedIndex = xx Then
            Me.MenuItemMenuSdp.Enabled = True
            Me.MenuItemMenuSetService.Enabled = False
        Else
            Me.MenuItemMenuSdp.Enabled = False
            Me.MenuItemMenuSetService.Enabled = True
        End If
    End Sub

#End Region '"SDP"

    '===========================================================================
#Region "Client"
    Private m_cli As BluetoothClient
    Private m_lockCli As New Object 'To apply 'volatile' access to the m_devices shared field.
    '
    Private m_strm As System.Net.Sockets.NetworkStream
    Private m_wtr As System.IO.StreamWriter
    Private m_encoding As System.Text.Encoding
    Private m_connecting As Boolean
    Private m_disconnecting As Boolean

    '--------------------------------------------------------------
    Sub Form1Connect_Load(ByVal sender As System.Object, ByVal e As System.EventArgs)        ''====
        Me.ComboBoxServices.ValueMember = "Value"
        Me.ComboBoxServices.DisplayMember = "Name"
        Me.ComboBoxServices.DataSource = BluetoothServiceItem.GetWellKnownServices
        Me.ComboBoxServices.SelectedItem = New BluetoothServiceItem(BluetoothService.SerialPort, "tmp")
        '
        If Environment.OSVersion.Platform = PlatformID.Win32NT _
        OrElse Environment.OSVersion.Platform = PlatformID.Win32Windows Then
            ' Only the full framework supports encoding x-IA5, NETCF and Mono don't.
            comboBoxEncoding.SelectedIndex = 0
        Else
            comboBoxEncoding.SelectedIndex = 3
        End If
        '
        labelState.Text = "Disconnected"
        labelSendPduLength.Text = ""
        '
        m_cli = New BluetoothClient
    End Sub

    '--------------------------------------------------------------
    Private Sub newBluetoothClient()
        m_connecting = False
        m_disconnecting = True
        newBluetoothClientUpdateLabelStatus("Disconnecting...")
        If Not m_wtr Is Nothing Then
            'If m_wtr.BaseStream IsNot Nothing Then
            'If m_wtr.BaseStream.CanRead OrElse m_wtr.BaseStream.CanWrite Then
            m_wtr.Close()
            'End If
            ' Closing the writer should have closed this too.
            System.Diagnostics.Debug.Assert(Not m_cli.Connected)
            ' But we'll close anyway...
        End If
        m_cli.Close()
        m_cli = New BluetoothClient()
        newBluetoothClientUpdateLabelStatus("Disconnected")
    End Sub
    Sub newBluetoothClientUpdateLabelStatus(ByVal text As String)
        If Me.labelState.InvokeRequired Then
            Dim d As ReceiveAppendCallback = New ReceiveAppendCallback(AddressOf newBluetoothClientUpdateLabelStatus)
            Me.Invoke(d, New Object() {text})
        Else
            labelState.Text = text
        End If
    End Sub

    '--------------------------------------------------------------
    Sub Form1Connect_FormClosing()
        m_disconnecting = True
        If Not m_wtr Is Nothing Then
            m_wtr.Close()
            ' Closing the writer should have closed this too.
            System.Diagnostics.Debug.Assert(Not m_cli.Connected)
            ' But we'll close anyway...
        End If
        If Not m_cli Is Nothing Then
            m_cli.Close()
        End If
    End Sub

    '--------------------------------------------------------------
    Private Sub ButtonDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonDisconnect.Click
        If m_connecting Then
            ' During asynchronous Connect let the disconnect button be used 
            ' to cancel the connect attempt.  We must not delete the extant 
            ' IrDAClient until the connect attempt has completed (so don't
            ' use newIrdaClient()). It will instead be called in the 
            ' EndConnect callback when the cancel is actioned.
            m_cli.Close()
        Else
            m_disconnecting = True
            'labelState.Text = "Disconnecting..."
            newBluetoothClient()
            'labelState.Text = "Disconnected"
        End If
    End Sub

    Sub zzzzDump(ByVal obj As Object)
        Dim type As String
        If obj Is Nothing Then
            type = "null"
        Else
            type = obj.GetType().Name
        End If
        Console.WriteLine("[" & type + "]: " & zzzzToString(obj))
    End Sub


    Function zzzzToString(ByVal obj As Object) As String
        If obj Is Nothing Then
            Return "null"
        Else
            Return obj.ToString()
        End If
    End Function

    ''==
    Function GetSelectedService() As Guid
        Dim svc As Guid
        '
        If Not Me.ComboBoxServices.SelectedItem Is Nothing Then
            Dim btStdSvc As BluetoothServiceItem
            btStdSvc = CType(Me.ComboBoxServices.SelectedItem, BluetoothServiceItem)
            svc = btStdSvc.Uuid
            Return svc
        End If
        '
        Dim svcText As String = Me.ComboBoxServices.Text
        Try
            svc = New Guid(svcText) ' Parse!
            Return svc
        Catch ofex As OverflowException
        Catch fmtex As FormatException
            ' TODO if users types space e.g. on selecting a standard service then 
            ' SelectedItem is null, we should find the last whitespace-separated
            ' piece of text and try and parse it...
        End Try
        Return Guid.Empty
    End Function
    ''==


    Private Sub ButtonTestDiscoveryUi_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonTestDiscoveryUi.Click, SelectDialogToolStripMenuItem.Click
        'Dim cod As ClassOfDevice = New ClassOfDevice(1776)
        ''
        ''
        'InTheHand.Windows.Forms.SelectBluetoothDeviceDialog.Internal___useOwnImpl = True
        Dim x As New InTheHand.Windows.Forms.SelectBluetoothDeviceDialog()
        Dim rslt As DialogResult = x.ShowDialog()
        Console.WriteLine("sbdd.ShowDialog returned")
        Dim txt As String = "rslt: " & rslt.ToString() & ", device: "
        If x.SelectedDevice Is Nothing Then
            txt &= "<null>"
        Else
            txt &= x.SelectedDevice.DeviceName
        End If
        MessageBox_Show(Me, txt)
    End Sub


    Private Sub ButtonSetSvcF_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSetSvcF.Click, MenuItemSetSvcFToolStripMenuItem.Click
        SetServiceState(False)
    End Sub
    Private Sub ButtonSetSvcT_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSetSvcT.Click, MenuItemSetSvcTToolStripMenuItem.Click
        SetServiceState(True)
    End Sub
    Private Sub SetServiceState(ByVal state As Boolean)
        '----------------------------------------------
        ' Gather input from user controls
        '----------------------------------------------
        ' Device
        Dim selectedDevice As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If selectedDevice Is Nothing Then Exit Sub
        ' Service
        Dim svc As Guid = GetSelectedService()
        If Guid.Empty.Equals(svc) Then
            Beep()
            MessageBox_Show(Me, "Invalid Service UUID.")
            Return
        End If
        '--------
        Try
            selectedDevice.SetServiceState(svc, state, True)
        Catch ex As System.ComponentModel.Win32Exception
            MessageBox_Show(Me, ex.ToString(), "SetServiceState")
        End Try
    End Sub

    Private Sub ButtonConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonConnect.Click
        If m_cli.Connected Then
            MessageBox_Show(Me, "Already connected.")
            Return
        End If
        If m_connecting Then
            Beep()
            MessageBox_Show(Me, "Already connected..")
            Return
        End If
        '----------------------------------------------
        ' Gather input from user controls
        '----------------------------------------------
        ' Device
        Dim selectedDevice As BluetoothDeviceInfo = GetSelectedDeviceInfo()
        If selectedDevice Is Nothing Then Exit Sub
        ' Service
        Dim svc As Guid = GetSelectedService()
        If Guid.Empty.Equals(svc) Then
            Beep()
            MessageBox_Show(Me, "Invalid Service UUID.")
            Return
        End If
        ' Settings
        System.Diagnostics.Debug.Assert(Not m_cli.Authenticate, "should have not been set directly...")
        If Me.CheckBoxAuthenticate.Checked Then
            m_cli.Authenticate = Me.CheckBoxAuthenticate.Checked
        End If
        System.Diagnostics.Debug.Assert(Not m_cli.Encrypt, "should have not been set directly...")
        If Me.CheckBoxEncrypt.Checked Then
            m_cli.Encrypt = Me.CheckBoxEncrypt.Checked
        End If
        If Me.CheckBoxUsePin.Checked Then
            BluetoothSecurity.SetPin(selectedDevice.DeviceAddress, TextBoxPin.Text)
        End If
        '
        Try
            m_encoding = System.Text.Encoding.GetEncoding(comboBoxEncoding.Text)
        Catch ex As ArgumentException
            MessageBox_Show(Me, "Unknown Encoding.")
            newBluetoothClient()
            Return
        End Try
        '
        '----------------------------------------------
        ' Prepare the UI
        '----------------------------------------------
        textBoxReceive.Text = String.Empty

        '----------------------------------------------
        ' Connect etc
        '----------------------------------------------
        m_disconnecting = False
        labelState.Text = "Connecting..."
        Try
#If NON_ASYNC_CONNECT Then
            m_cli.Connect(selectedDevice.DeviceAddress, svc)
            DoneConnect()
#Else
            Dim cbk As AsyncCallback = New AsyncCallback(AddressOf ConnectCallback)
            m_connecting = True
            m_cli.BeginConnect(selectedDevice.DeviceAddress, svc, cbk, Nothing)
#End If
        Catch sex As System.Net.Sockets.SocketException
            Dim msg As String = "Connect failed: " _
                    + sex.SocketErrorCode.ToString() _
                    + " (" + sex.SocketErrorCode.ToString("D") + "); " _
                    + sex.Message
            MessageBox_Show(Me, msg)
            labelState.Text = msg
            newBluetoothClient()
            Return
        End Try
    End Sub

    Sub ConnectCallback(ByVal ar As IAsyncResult)
        Dim uiUpdate As New AsyncCallback(AddressOf ConnectCallback__Invokee)
        If labelState.InvokeRequired Then
            labelState.Invoke(uiUpdate, ar)
        Else
            uiUpdate(ar)
        End If
    End Sub
    Private Sub ConnectCallback__Invokee(ByVal ar As IAsyncResult)
        'DLGTDim uiUpdate As EventHandler
        Try
            m_cli.EndConnect(ar)
            m_connecting = False
            ' Using goto is ok in error handling situations...
            GoTo connectSuccess
        Catch nrex As NullReferenceException
            ' Close() on IrDAClient sets its socket to null, so EndConnect throws...
            'System.Diagnostics.Debug.Assert(nrex.Message == "foo")
            'DLGTuiUpdate = delegate {
            labelState.Text = "Connect cancelled."
            'DLGT}
        Catch odEx As ObjectDisposedException
            ' Just before the client's internal socket instance is set to null 
            ' (see the catch above) is is Disposes so catch the resultant error.
            'DLGTuiUpdate = delegate {
            labelState.Text = "Connect cancelled."
            'DLGT}
        Catch sex As System.Net.Sockets.SocketException
            'DLGTuiUpdate = delegate {
            Dim msg As String = "Connect failed: " _
                + sex.SocketErrorCode.ToString() _
                + " (" + sex.SocketErrorCode.ToString("D") + ") " _
                + sex.Message
            MessageBox_Show(Me, msg)
            labelState.Text = msg
            'DLGT}
        End Try
        ' Dropped out of one of the exception handlers, exit after updating...
        'UI thread-safe
        'DLGTIf labelState.InvokeRequired Then
        'DLGT    labelState.Invoke(uiUpdate)
        'DLGTElse
        'DLGT    uiUpdate(labelState, Nothing)
        'DLGTEnd If
        newBluetoothClient()
        Return
        '-------------------------------
        ' Successful Connection.
connectSuccess:
        'Dim selectedMode as IrProtocol = CType(ar.AsyncState, IrProtocol)
        DoneConnect()
    End Sub

    Private Sub DoneConnect()
        Dim uiUpdate As New EventHandler(AddressOf DoneConnect__Invokee)
        If labelState.InvokeRequired Then
            labelState.Invoke(uiUpdate)
        Else
            uiUpdate(Me, New EventArgs())
        End If
    End Sub
    Private Sub DoneConnect__Invokee(ByVal sender As Object, ByVal e As EventArgs)
        m_strm = m_cli.GetStream()
        m_wtr = New System.IO.StreamWriter(m_strm, m_encoding)

        '----------------------------------------------
        ' Update the UI
        '----------------------------------------------
        'DLGTDim uiUpdate as EventHandler = delegate {
        labelState.Text = "Connected"
        ' Display the maximum send size where relevant.
        'if selectedMode == IrProtocol.IrLMP then
        '    Dim objValue As Object = m_cli.Client.GetSocketOption( _
        '        IrDASocketOptionLevel.IrLmp, IrDASocketOptionName.SendPduLength)
        '    int sendPduLength = (int)objValue
        '    labelSendPduLength.Text = sendPduLength.ToString()
        '    else {
        'labelSendPduLength.Text = "N/A"
        '    }
        textBoxSend.Focus()
        'DLGT}
        'UI thread-safe
        'DLGTIf labelState.InvokeRequired Then
        'DLGT    labelState.Invoke(uiUpdate)
        'DLGTElse
        'DLGT    uiUpdate(Me, New EventArgs())
        'DLGTEnd If

        '----------------------------------------------
        ' Start the receive thread.
        '----------------------------------------------
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf receiveThreadFn)
    End Sub


    Private Sub CheckBoxAuthenticate_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxAuthenticate.CheckedChanged
        If (Not m_cli Is Nothing) AndAlso m_cli.Connected Then
            m_cli.Authenticate = Me.CheckBoxAuthenticate.Checked
        End If
    End Sub

    Private Sub CheckBoxEncrypt_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxEncrypt.CheckedChanged
        If (Not m_cli Is Nothing) AndAlso m_cli.Connected Then
            m_cli.Encrypt = Me.CheckBoxEncrypt.Checked
        End If
    End Sub

    Private Sub CheckBoxUsePin_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxUsePin.CheckedChanged
        Debug.Assert(Me.CheckBoxUsePin.Checked <> Me.TextBoxPin.Enabled)
        Me.TextBoxPin.Enabled = Me.CheckBoxUsePin.Checked
        ' When connected, set/reset the Pin immediately
        If (Not m_cli Is Nothing) AndAlso m_cli.Connected Then
            If Me.TextBoxPin.Enabled Then
                BluetoothSecurity.SetPin(CType(m_cli.Client.RemoteEndPoint, BluetoothEndPoint).Address, TextBoxPin.Text)
            Else
                'm_cli.Pin = Nothing
            End If
        End If
    End Sub

    ''------------------------------------------------------------------
    Private Sub buttonSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonSend.Click
        If m_wtr Is Nothing _
                OrElse m_wtr.BaseStream Is Nothing _
                OrElse Not m_wtr.BaseStream.CanWrite Then
            MessageBox_Show(Me, "Not connected.")
        Else
            ' Assume v.25ter, so use a CR.
            Dim str As String = textBoxSend.Text + vbCr
            Try
                m_wtr.Write(str)
                m_wtr.Flush()
            Catch ioex As System.IO.IOException
                Dim sex As SocketException = TryCast(ioex.InnerException, SocketException)
                If Not sex Is Nothing Then
                    Dim Err As SocketError = sex.SocketErrorCode
                    receiveAppend("!! Send SocketException: " _
                        + Err.ToString() + " (" + Err.ToString("D") _
                        + ") " + ioex.Message)
                Else
                    receiveAppend("!! Send IOException: " + ioex.Message)
                End If
                newBluetoothClient()
            End Try
        End If
    End Sub

    Sub receiveThreadFn(ByVal state As Object)
        receiveThreadFn()
    End Sub
    Sub receiveThreadFn()
        If Not m_cli.Connected _
                OrElse m_strm Is Nothing _
                OrElse Not m_strm.CanRead Then
            Beep()
            Return
        End If

        Dim rdr As System.IO.StreamReader = New System.IO.StreamReader(m_strm, m_encoding)
        Dim buf(99) As Char
        Try
            While True
                ' We don't use ReadLine because we then don't get to see the 
                ' CR and LF characters.  And we often get the series \r\r\n 
                ' which should appear as one new line, but would appear as two 
                ' if we did textBox.Append("\n") each ReadLine.
                Dim numRead As Integer = rdr.Read(buf, 0, buf.Length)
                If numRead = 0 Then
                    newBluetoothClient()
                    Return
                End If
                Dim str As String = New String(buf, 0, numRead)
                receiveAppend(str)
            End While
        Catch ioex As System.IO.IOException
            If Not m_disconnecting Then
                Dim sex As SocketException = TryCast(ioex.InnerException, SocketException)
                If Not sex Is Nothing Then
                    Dim Err As SocketError = sex.SocketErrorCode
                    receiveAppend("!! SocketException: " _
                            + Err.ToString() + " (" + Err.ToString("D") _
                            + ") " + ioex.Message)
                Else
                    receiveAppend("!! IOException: " + ioex.Message)
                End If
            End If
            newBluetoothClient()
        End Try
    End Sub

    Delegate Sub ReceiveAppendCallback(ByVal str As String)

    ' UI thread-safe updating.
    Sub receiveAppend(ByVal str As String)
        If Me.textBoxReceive.InvokeRequired Then
            Dim d As ReceiveAppendCallback = New ReceiveAppendCallback(AddressOf receiveAppend)
            Me.Invoke(d, New Object() {str})
        Else
            textBoxReceive.AppendText(str)
        End If
    End Sub

#End Region '"Client"

    '===========================================================================
#Region "Server"
    Private m_serverListener As BluetoothListener
    Private m_serverCli As BluetoothClient
    'Private m_serverLockCli As New Object 'To apply 'volatile' access to the m_devices shared field.
    '
    Private m_serverStrm As System.Net.Sockets.NetworkStream
    Private m_serverWtr As System.IO.StreamWriter
    Private m_serverEncoding As System.Text.Encoding
    Private m_serverAccepting As Boolean
    Private m_serverDisconnecting As Boolean
    '
    ' Its a [Flag] attribute, so multiple value can be combined
    Private m_selectedScs As ServiceClass
    Dim m_ServerCodServiceClassesDialog As New FormSelectServiceClasses

    '--------------------------------------------------------------
    Sub newBluetoothServer()
        newBluetoothServerUpdateLabelStatus("Disconnecting...")
        If Not m_serverListener Is Nothing Then
            m_serverListener.Stop()
        End If
        If Not m_serverCli Is Nothing Then
            m_serverCli.Close()
            ' After Close() calling Connect fails with NRE, so delete the client itself.
            m_serverCli = Nothing
        End If
        newBluetoothServerUpdateLabelStatus("Disconnected")
    End Sub
    Sub newBluetoothServerUpdateLabelStatus(ByVal text As String)
        If Me.labelState.InvokeRequired Then
            Dim d As ReceiveAppendCallback = New ReceiveAppendCallback(AddressOf newBluetoothServerUpdateLabelStatus)
            Me.Invoke(d, New Object() {text})
        Else
            LabelServerState.Text = text
        End If
    End Sub

    '--------------------------------------------------------------
    Sub Form1Server_Load(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Me.ComboBoxServerServices.ValueMember = "Value"
        Me.ComboBoxServerServices.DisplayMember = "Name"
        Me.ComboBoxServerServices.DataSource = BluetoothServiceItem.GetWellKnownServices
        Me.ComboBoxServerServices.SelectedItem = New BluetoothServiceItem(BluetoothService.SerialPort, "tmp")
        '
        ComboBoxServerEncoding.SelectedIndex = 0
        '
        LabelServerState.Text = "Disconnected"
        '
        m_serverCli = New BluetoothClient
        RefreshServerCodServicesUi()
    End Sub

    '--------------------------------------------------------------
    Function GetServerSelectedService() As Guid
        Dim svc As Guid
        '
        If Me.ComboBoxServerServices.SelectedItem IsNot Nothing Then
            Dim btStdSvc As BluetoothServiceItem
            btStdSvc = CType(Me.ComboBoxServerServices.SelectedItem, BluetoothServiceItem)
            svc = btStdSvc.Uuid
            Return svc
        End If
        '
        Dim svcText As String = Me.ComboBoxServerServices.Text
        Try
            svc = New Guid(svcText) ' Parse!
            Return svc
        Catch ofex As OverflowException
        Catch fmtex As FormatException
            ' TODO if users types space e.g. on selecting a standard service then 
            ' SelectedItem is null, we should find the last whitespace-separated
            ' piece of text and try and parse it...
        End Try
        Return Guid.Empty
    End Function

    Private Sub ButtonServerListen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonServerListen.Click
        If (Not m_serverCli Is Nothing) AndAlso m_serverCli.Connected Then
            MessageBox_Show(Me, "Already connected.")
            Return
        End If
        If m_serverAccepting Then
            Beep()
            MessageBox_Show(Me, "Already connected..")
            Return
        End If
        '----------------------------------------------
        ' Gather input from user controls
        '----------------------------------------------
        ' Service
        Dim svc As Guid = GetServerSelectedService()
        If Guid.Empty.Equals(svc) Then
            Beep()
            MessageBox_Show(Me, "Invalid Service UUID.")
            Return
        End If
        '
        m_serverDisconnecting = False
        LabelServerState.Text = "Listening..."
        Dim phase As String = "Pre!"
        phase = "Init"
        m_serverListener = New BluetoothListener(svc)
        m_serverListener.ServiceClass = m_selectedScs
        ' Settings
        System.Diagnostics.Debug.Assert(Not m_serverListener.Authenticate, "should have not been set directly...")
        If Me.CheckBoxServerAuthenticate.Checked Then
            m_serverListener.Authenticate = Me.CheckBoxServerAuthenticate.Checked
        End If
        System.Diagnostics.Debug.Assert(Not m_serverListener.Encrypt, "should have not been set directly...")
        If Me.CheckBoxServerEncrypt.Checked Then
            m_serverListener.Encrypt = Me.CheckBoxServerEncrypt.Checked
        End If
        If Me.CheckBoxUsePin.Checked Then
            'TODO m_serverListener.Pin = Me.TextBoxServerPin.Text
        End If
        '
        Try
            m_serverEncoding = System.Text.Encoding.GetEncoding(ComboBoxServerEncoding.Text)
        Catch ex As ArgumentException
            MessageBox_Show(Me, "Unknown Encoding.")
            newBluetoothServer()
            Return
        End Try
        '
        '----------------------------------------------
        ' Prepare the UI
        '----------------------------------------------
        TextBoxServerReceive.Clear()

        '----------------------------------------------
        ' Connect etc
        '----------------------------------------------
        'm_serverDisconnecting = False
        'LabelServerState.Text = "Connecting..."
        'Dim phase As String = "Pre!"
        Try
            'phase = "Init"
            'm_serverListener = New BluetoothListener(svc)
            phase = "Bind"
            m_serverListener.Start()
            '
            phase = "Begin Accept"
            Dim cbk As AsyncCallback = New AsyncCallback(AddressOf ServerAcceptCallback)
            m_serverAccepting = True
            m_serverListener.BeginAcceptBluetoothClient(cbk, Nothing)
        Catch sex As System.Net.Sockets.SocketException
            Dim msg As String = phase + " failed: " _
                    + sex.SocketErrorCode.ToString() _
                    + " (" + sex.SocketErrorCode.ToString("D") + "); " _
                    + sex.Message
            MessageBox_Show(Me, msg)
            LabelServerState.Text = msg
            newBluetoothServer()
            Return
        End Try
    End Sub

    Private Sub ButtonServerDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonServerDisconnect.Click
        m_serverDisconnecting = True
        newBluetoothServer()
    End Sub

    Sub ServerAcceptCallback(ByVal ar As IAsyncResult)
        Dim uiUpdate As New AsyncCallback(AddressOf ServerAcceptCallback__Invokee)
        If labelState.InvokeRequired Then
            labelState.Invoke(uiUpdate, ar)
        Else
            uiUpdate(ar)
        End If
    End Sub
    Private Sub ServerAcceptCallback__Invokee(ByVal ar As IAsyncResult)
        'DLGTDim uiUpdate As EventHandler
        Try
            m_serverAccepting = False
            m_serverCli = m_serverListener.EndAcceptBluetoothClient(ar)
            ' Using goto is ok in error handling situations...
            GoTo acceptSuccess
        Catch argex As ArgumentException
            If m_serverDisconnecting Then
                ' When lsnr.Stop() is called, it creates a new socket internally and
                ' therefor calling EndAccept fails...
                Debug.Assert(argex.Message.StartsWith( _
                        "The IAsyncResult object was not returned from the corresponding asynchronous method on this class."), _
                    "Unexpected ArgEx")
            Else
                Throw
            End If
            'Catch nrex As NullReferenceException
            '    ' Close() on IrDAClient sets its socket to null, so EndConnect throws...
            '    'System.Diagnostics.Debug.Assert(nrex.Message == "foo")
            '    'DLGTuiUpdate = delegate {
            '    LabelServerState.Text = "Connect cancelled."
            '    'DLGT}
            'Catch odEx As ObjectDisposedException
            '    ' Just before the client's internal socket instance is set to null 
            '    ' (see the catch above) is is Disposes so catch the resultant error.
            '    'DLGTuiUpdate = delegate {
            '    LabelServerState.Text = "Listening cancelled."
            '    'DLGT}
        Catch sex As System.Net.Sockets.SocketException
            'DLGTuiUpdate = delegate {
            Dim msg As String = "Accept failed: " _
                + sex.SocketErrorCode.ToString() _
                + " (" + sex.SocketErrorCode.ToString("D") + ") " _
                + sex.Message
            MessageBox_Show(Me, msg)
            LabelServerState.Text = msg
            'DLGT}
        End Try
        ' Dropped out of one of the exception handlers, exit after updating...
        'UI thread-safe
        'DLGTIf labelState.InvokeRequired Then
        'DLGT    labelState.Invoke(uiUpdate)
        'DLGTElse
        'DLGT    uiUpdate(labelState, Nothing)
        'DLGTEnd If
        newBluetoothServer()
        Return
        '-------------------------------
        ' Successful Connection.
acceptSuccess:
        ' Just accept one connection then stop listening.
        m_serverListener.Stop()
        m_serverListener = Nothing
        ServerDoneAccept()
    End Sub

    Private Sub ServerDoneAccept()
        Dim uiUpdate As New EventHandler(AddressOf ServerDoneAccept__Invokee)
        If LabelServerState.InvokeRequired Then
            LabelServerState.Invoke(uiUpdate)
        Else
            uiUpdate(Me, New EventArgs())
        End If
    End Sub
    Private Sub ServerDoneAccept__Invokee(ByVal sender As Object, ByVal e As EventArgs)
        m_serverStrm = m_serverCli.GetStream()
        m_serverWtr = New System.IO.StreamWriter(m_serverStrm, m_serverEncoding)

        '----------------------------------------------
        ' Update the UI
        '----------------------------------------------
        'DLGTDim uiUpdate as EventHandler = delegate {
        LabelServerState.Text = "Connected"
        TextBoxServerSend.Focus()
        'DLGT}
        'UI thread-safe
        'DLGTIf labelState.InvokeRequired Then
        'DLGT    labelState.Invoke(uiUpdate)
        'DLGTElse
        'DLGT    uiUpdate(Me, New EventArgs())
        'DLGTEnd If

        '----------------------------------------------
        ' Start the receive thread.
        '----------------------------------------------
        System.Threading.ThreadPool.QueueUserWorkItem(AddressOf serverReceiveThreadFn)
    End Sub


    Private Sub CheckBoxServerAuthenticate_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxServerAuthenticate.CheckedChanged
        If m_serverCli IsNot Nothing AndAlso m_serverCli.Connected Then
            m_serverCli.Authenticate = Me.CheckBoxServerAuthenticate.Checked
        End If
    End Sub

    Private Sub CheckBoxServerEncrypt_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxServerEncrypt.CheckedChanged
        If m_serverCli IsNot Nothing AndAlso m_serverCli.Connected Then
            m_serverCli.Encrypt = Me.CheckBoxServerEncrypt.Checked
        End If
    End Sub

    Private Sub CheckBoxServerUsePin_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxServerUsePin.CheckedChanged
        Debug.Assert(Me.CheckBoxServerUsePin.Checked <> Me.TextBoxServerPin.Enabled)
        Me.TextBoxServerPin.Enabled = Me.CheckBoxServerUsePin.Checked
        ' When listening, set/reset the Pin immediately
        If (Not m_serverListener Is Nothing) AndAlso m_serverListener.Pending Then
            If Me.TextBoxPin.Enabled Then
                'TODO m_serverListener.Pin = Me.TextBoxPin.Text
            Else
                'TODO m_serverListener.Pin = Nothing
            End If
        End If
        ' When connected, set/reset the Pin immediately
        If (Not m_serverCli Is Nothing) AndAlso m_serverCli.Connected Then
            If Me.TextBoxPin.Enabled Then
                BluetoothSecurity.SetPin(CType(m_serverCli.Client.RemoteEndPoint, BluetoothEndPoint).Address, TextBoxPin.Text)
            Else
                'm_serverCli.Pin = Nothing
            End If
        End If
    End Sub

    ''------------------------------------------------------------------
    Private Sub buttonServerSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonServerSend.Click
        If m_serverWtr Is Nothing _
                OrElse m_serverWtr.BaseStream Is Nothing _
                OrElse Not m_serverWtr.BaseStream.CanWrite Then
            MessageBox_Show(Me, "Not connected.")
        Else
            ' Assume v.25ter, so use a CR.
            Dim str As String = TextBoxServerSend.Text + vbCr
            Try
                m_serverWtr.Write(str)
                m_serverWtr.Flush()
            Catch ioex As System.IO.IOException
                Dim sex As SocketException = TryCast(ioex.InnerException, SocketException)
                If sex IsNot Nothing Then
                    Dim Err As SocketError = sex.SocketErrorCode
                    serverReceiveAppend("!! Send SocketException: " _
                        + Err.ToString() + " (" + Err.ToString("D") _
                        + ") " + ioex.Message)
                Else
                    serverReceiveAppend("!! Send IOException: " + ioex.Message)
                End If
                newBluetoothServer()
            End Try
        End If
    End Sub

    Sub serverReceiveThreadFn(ByVal state As Object)
        serverReceiveThreadFn()
    End Sub
    Sub serverReceiveThreadFn()
        If Not m_serverCli.Connected _
                OrElse m_serverStrm Is Nothing _
                OrElse Not m_serverStrm.CanRead Then
            Beep()
            Return
        End If
        '
#If DEBUG Then
        System.Threading.Thread.CurrentThread.Name = "serverReceiveThreadFn"
#End If
        '
        Dim rdr As System.IO.StreamReader = New System.IO.StreamReader(m_serverStrm, m_serverEncoding)
        Dim buf(99) As Char
        Try
            While True
                ' We don't use ReadLine because we then don't get to see the 
                ' CR and LF characters.  And we often get the series \r\r\n 
                ' which should appear as one new line, but would appear as two 
                ' if we did textBox.Append("\n") each ReadLine.
                Dim numRead As Integer = rdr.Read(buf, 0, buf.Length)
                If numRead = 0 Then
                    newBluetoothServer()
                    Return
                End If
                Dim str As String = New String(buf, 0, numRead)
                serverReceiveAppend(str)
                If CheckBoxServerLoopback.Checked Then
                    ' !! What's the thread-safety of this?
                    m_serverWtr.Write(buf, 0, numRead)
                    m_serverWtr.Flush()
                End If
            End While
        Catch ioex As System.IO.IOException
            If Not m_serverDisconnecting Then
                Dim sex As SocketException = TryCast(ioex.InnerException, SocketException)
                If sex IsNot Nothing Then
                    Dim Err As SocketError = sex.SocketErrorCode
                    serverReceiveAppend("!! SocketException: " _
                            + Err.ToString() + " (" + Err.ToString("D") _
                            + ") " + ioex.Message)
                Else
                    serverReceiveAppend("!! IOException: " + ioex.Message)
                End If
            End If
            newBluetoothClient()
        End Try
    End Sub

    'Delegate Sub ReceiveAppendCallback(ByVal str As String)

    ' UI thread-safe updating.
    Sub serverReceiveAppend(ByVal str As String)
        If Me.textBoxReceive.InvokeRequired Then
            Dim d As ReceiveAppendCallback = New ReceiveAppendCallback(AddressOf serverReceiveAppend)
            Me.Invoke(d, New Object() {str})
        Else
            TextBoxServerReceive.AppendText(str)
        End If
    End Sub

    Private Sub TextBoxServerCodServices_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxServerCodServices.DoubleClick
        m_ServerCodServiceClassesDialog.SelectedServiceClasses = m_selectedScs
        Dim rslt As DialogResult = m_ServerCodServiceClassesDialog.ShowDialog(Me)
        'MessageBox.Show( _
        '    String.Format("rslt: {0}, val: {1}", rslt, m_ServerCodServiceClassesDialog.SelectedServiceClasses))
        If rslt = Windows.Forms.DialogResult.OK Then
            m_selectedScs = m_ServerCodServiceClassesDialog.SelectedServiceClasses
        End If
        RefreshServerCodServicesUi()
    End Sub

    Sub RefreshServerCodServicesUi()
        If m_selectedScs.ToString() = "Unknown" Then 'hack m_selectedScs.ToString() = "Unknown"
            Me.TextBoxServerCodServices.Text = "None" & " ..."
        Else
            Me.TextBoxServerCodServices.Text = m_selectedScs.ToString() + " ..."
        End If
    End Sub

#End Region '"Server"

End Class
