Imports InTheHand.Net.Bluetooth
Imports InTheHand.Net
Imports system.io
Imports InTheHand.Net.sockets
Imports InTheHand.Net.Bluetooth.AttributeIds

Module Module1

    Public Sub Main()
        Documentation.ObexPutFile()
        Documentation.ObexPutData()
        'Documentation.DisplayBluetoothRadio(Console.Out)
        Misc.GetPrimaryRadio()
        Misc.Discovery()
        Misc.SelectBluetoothDeviceDialog()
        '
        Misc.ObexListen()
        '
        Dim inst As New Tests()
        inst.Win32AuthCallbackTwoSeparateAuthentications()
        inst.Win32AuthCallbackInitialBadPasscodeAndRetry()
        '
        Misc.ObexPushAddressString("127.0.0.1")
        Misc.ObexPushAddressString("000a3a6865bb")
        Misc.ObexPushBluetoothUi()
    End Sub
End Module

Class Documentation
    ' ==== Bluetooth Radio ====
    Public Shared Sub DisplayBluetoothRadio(ByVal wtr As System.IO.TextWriter)
        Dim myRadio As BluetoothRadio = BluetoothRadio.PrimaryRadio
        If myRadio Is Nothing Then
            wtr.WriteLine("No radio hardware or unsupported software stack")
            Return
        End If
        myRadio.Mode = RadioMode.PowerOff
        Dim mode As RadioMode = myRadio.Mode
        wtr.WriteLine("* Radio, address: {0:C}")
        wtr.WriteLine("Mode: " & mode.ToString())
        wtr.WriteLine("Name: " & myRadio.Name & ", LmpSubversion: " & myRadio.LmpSubversion)
        wtr.WriteLine("ClassOfDevice: " & myRadio.ClassOfDevice.ToString() _
            & ", device: " & myRadio.ClassOfDevice.Device.ToString() _
            & " / service: " & myRadio.ClassOfDevice.Service.ToString())
        '
        '
        ' Enable discoverable mode
#If False Then
        wtr.WriteLine()
        myRadio.Mode = RadioMode.Discoverable
        wtr.WriteLine("Radio Mode now: " & myRadio.Mode.ToString())
#End If
    End Sub

    Public Shared Sub ObexPutFile()
        ' The host part of the URI is the device address, e.g. IrDAAddress.ToString(),
        ' and the file part is the OBEX object name.
        Dim uri As New Uri("obex://localhost/HelloWorld.txt")
        Dim req As New ObexWebRequest(uri)
        req.ReadFile("Hello World.txt")
        Dim rsp As ObexWebResponse = CType(req.GetResponse(), ObexWebResponse)
        Console.WriteLine("Response Code: {0} (0x{0:X})", rsp.StatusCode)
    End Sub

    Public Shared Sub ObexPutData()
        ' The host part of the URI is the device address, e.g. IrDAAddress.ToString(),
        ' and the file part is the OBEX object name.
        Dim uri As New Uri("obex://127.0.0.1/HelloWorld2.txt")
        Dim req As New ObexWebRequest(uri)
        Using content As Stream = req.GetRequestStream()
            ' Using a StreamWriter to write text to the stream...
            Using wtr As New StreamWriter(content)
                wtr.WriteLine("Hello World GetRequestStream")
                wtr.WriteLine("Hello World GetRequestStream 2")
                wtr.Flush()
                ' Set the Length header value
                req.ContentLength = content.Length
            End Using
        End Using
        Dim rsp As ObexWebResponse = CType(req.GetResponse(), ObexWebResponse)
        Console.WriteLine("Response Code: {0} (0x{0:X})", rsp.StatusCode)
    End Sub

    ' ==== Bluetooth SDP ====
    Sub SdpListenerRecord()
        Dim serviceClassUuid As New Guid("{D91ED51C-B8A1-4ce7-B76A-3637327C5552}")
        '
        '
        Dim pdl As ServiceElement = ServiceRecordHelper.CreateRfcommProtocolDescriptorList()
        Dim classList As ServiceElement = New ServiceElement(ElementType.ElementSequence, _
          New ServiceElement(ElementType.Uuid128, serviceClassUuid))
        Dim recordXX As ServiceRecord = New ServiceRecord( _
          New ServiceAttribute(UniversalAttributeId.ServiceClassIdList, classList), _
          New ServiceAttribute(UniversalAttributeId.ProtocolDescriptorList, pdl) _
        )
        '
        '
        Dim str As ServiceElement = New ServiceElement(ElementType.TextString, "hello world")
        Dim langBaseList As ServiceElement = CreateEnglishUtf8PrimaryLanguageServiceElement()
        '
        '...
        Dim record As ServiceRecord = New ServiceRecord( _
            New ServiceAttribute(UniversalAttributeId.ServiceClassIdList, classList), _
            New ServiceAttribute(UniversalAttributeId.ProtocolDescriptorList, pdl), _
 _
            New ServiceAttribute(UniversalAttributeId.LanguageBaseAttributeIdList, langBaseList), _
            New ServiceAttribute(ServiceRecord.CreateLanguageBasedAttributeId( _
                    UniversalAttributeId.ServiceName, _
                    LanguageBaseItem.PrimaryLanguageBaseAttributeId), _
            str) _
        )

    End Sub

    Private Shared Function CreateEnglishUtf8PrimaryLanguageServiceElement() As ServiceElement
        Dim englishUtf8PrimaryLanguage As ServiceElement = LanguageBaseItem.CreateElementSequenceFromList( _
                New LanguageBaseItem() { _
                    New LanguageBaseItem("en", LanguageBaseItem.Utf8EncodingId, _
                         LanguageBaseItem.PrimaryLanguageBaseAttributeId) _
        })
        Return englishUtf8PrimaryLanguage
    End Function

End Class

Class Misc

    '----------------
    Shared Sub GetPrimaryRadio()
        Console.WriteLine("---- GetPrimaryRadio ----")
        Dim myRadio As BluetoothRadio = BluetoothRadio.PrimaryRadio
        If myRadio Is Nothing Then
            Console.WriteLine("No radio hardware or unsupported software stack")
            Return
        End If
        Dim myAddr As BluetoothAddress = myRadio.LocalAddress
        Console.WriteLine("Local radio address is " & myAddr.ToString())
    End Sub

    '----------------
    Shared Sub Discovery()
        Console.WriteLine("---- Discovery ----")
        Dim deviceList() As BluetoothDeviceInfo = New BluetoothClient().DiscoverDevices()
        Dim len As Integer = deviceList.Length
        Console.WriteLine("found {0} devices", len)
    End Sub

    Shared Sub SelectBluetoothDeviceDialog()
        Console.WriteLine("---- SelectBluetoothDeviceDialog ----")
        Dim dlg As New InTheHand.Windows.Forms.SelectBluetoothDeviceDialog
        Dim rslt As System.Windows.Forms.DialogResult = dlg.ShowDialog()
        Console.WriteLine("dialog result: {0}", rslt)
        Console.WriteLine("selected device: {0}", dlg.SelectedDevice)
        If Not dlg.SelectedDevice Is Nothing Then
            Console.WriteLine("  address: {0}", dlg.SelectedDevice.DeviceAddress)
            Console.WriteLine("  name   : {0}", dlg.SelectedDevice.DeviceName)
        End If
    End Sub

    '----------------
    Shared Sub ObexListen()
        Dim lstnr As New ObexListener(ObexTransport.Bluetooth)
        lstnr.Start()
        Try
            Console.WriteLine("OBEX listening")
            Console.WriteLine("Hit Return to complete")
            Console.ReadLine()
        Finally
            lstnr.Stop()
        End Try
    End Sub

    '----------------
    Shared Sub ObexPushBluetoothUi()
        Dim dlg As New InTheHand.Windows.Forms.SelectBluetoothDeviceDialog
        Dim rslt As System.Windows.Forms.DialogResult = dlg.ShowDialog()
        If rslt = Windows.Forms.DialogResult.OK Then
            Dim device As BluetoothDeviceInfo = dlg.SelectedDevice
            Dim addr As String = device.DeviceAddress.ToString()
            ObexPushAddressString(addr)
        End If
    End Sub
    Shared Sub ObexPushAddressString(ByVal addressAsString As String)
        Dim uri As New Uri("obex-push://" & addressAsString & "/alan.txt")
        Dim owreq As ObexWebRequest = New ObexWebRequest(uri)
        Using reqContent As Stream = owreq.GetRequestStream()
            Using wtr As StreamWriter = New StreamWriter(reqContent)
                wtr.WriteLine("alan is cool")
            End Using
        End Using
        Dim rsp As ObexWebResponse = DirectCast(owreq.GetResponse(), ObexWebResponse)
    End Sub

End Class

Class Tests
    <TestFeature()> _
    Sub Win32AuthCallbackTwoSeparateAuthentications()
        Console.WriteLine("Authenticate two devices")
        Console.WriteLine("Passcode respectively: '{0}', '{1}', '{2}'", _
            "1234", "9876", "sdfghjkl")
        Win32AuthCallback__(AddressOf Win32AuthCallbackTwoSeparateAuthenticationsHandler)
    End Sub

    Private Win32AuthCallbackTwoSeparateAuthentications_count As Integer
    Sub Win32AuthCallbackTwoSeparateAuthenticationsHandler(ByVal sender As Object, ByVal e As InTheHand.Net.Bluetooth.BluetoothWin32AuthenticationEventArgs)
        Console.WriteLine("Authenticating {0} {1}", e.Device.DeviceAddress, e.Device.DeviceName)
        Console.WriteLine("  Attempt# {0}, Last error code {1}", e.AttemptNumber, e.PreviousNativeErrorCode)
        '
        If Win32AuthCallbackTwoSeparateAuthentications_count = 0 Then
            e.Pin = "1234"
        ElseIf Win32AuthCallbackTwoSeparateAuthentications_count = 1 Then
            e.Pin = "9876"
        ElseIf Win32AuthCallbackTwoSeparateAuthentications_count = 2 Then
            e.Pin = "sdfghjkl"
        End If
        Console.WriteLine("Using '{0}'", e.Pin)
        Win32AuthCallbackTwoSeparateAuthentications_count += 1
    End Sub

    Sub Win32AuthCallbackJohoHandler(ByVal sender As Object, ByVal e As InTheHand.Net.Bluetooth.BluetoothWin32AuthenticationEventArgs)
        Dim address As String = e.Device.DeviceAddress.ToString()
        Console.WriteLine("Received an authentication request from address " + address)

        ' compare the first 8 hex numbers, this is just a special case because in the
        ' used scenario the model of the devices can be identified by the first 8 hex
        ' numbers, the last 4 numbers being the device specific part.
        If address.Substring(0, 8).Equals("0099880D") OrElse _
                address.Substring(0, 8).Equals("0099880E") Then
            ' send authentication response
            e.Pin = "5276"
        ElseIf (address.Substring(0, 8).Equals("00997788")) Then
            ' send authentication response
            e.Pin = "1423"
        End If
    End Sub


    ' The output of a standard run is the following.  Note that the error code from
    ' the initial wrong response is error code 1244 which is ErrorNotAuthenticated.
    ' However when we try another attempt we get error code 1167 which is
    ' ErrorDeviceNotConnected.  So for the devices I've tested it's apparently not
    ' worth trying a second response.  If the user attempts to send another PIN-
    ' response (by setting the Pin field in the event args) then it will be ignored
    ' (we probably should have the event args throw when Pin is set in that case).
    ' 
    '[[
    ' Local radio address is 008099244999
    ' Authenticate two devices
    ' Passcode respectively: '1234', '9876', 'sdfghjkl'
    ' Making PC discoverable
    ' Hit Return to complete
    ' 
    ' Complete
    ' Authenticate one device -- with wrong passcode here the first two times.
    ' Passcode respectively: 'BAD-x', 'BAD-y', '9876'
    ' Making PC discoverable
    ' Hit Return to complete
    ' Authenticating 000A99686999 Win2kBelkin
    '   Attempt# 0, Last error code 0
    ' Using '0.88516242889927'
    ' Authenticating 000A99686999 Win2kBelkin
    '   Attempt# 1, Last error code 1244
    ' Using '0.59947050996100'
    ' Authenticating 000A99686999 Win2kBelkin
    '   Attempt# 2, Last error code 1167
    ' Using '9876'
    ']]
    <TestFeature()> _
    Sub Win32AuthCallbackInitialBadPasscodeAndRetry()
        Console.WriteLine("Authenticate one device -- with wrong passcode here the first two times.")
        Console.WriteLine("Passcode respectively: '{0}', '{1}', '{2}'", _
            "BAD-x", "BAD-y", "9876")
        Win32AuthCallback__(AddressOf Win32AuthCallbackInitialBadPasscodeAndRetryHandler)
    End Sub


    Sub Win32AuthCallback__(ByVal handler As System.EventHandler(Of BluetoothWin32AuthenticationEventArgs))
        Console.WriteLine("Making PC discoverable")
        Dim radio As InTheHand.Net.Bluetooth.BluetoothRadio = InTheHand.Net.Bluetooth.BluetoothRadio.PrimaryRadio
        Dim origRadioMode As InTheHand.Net.Bluetooth.RadioMode = radio.Mode
        radio.Mode = InTheHand.Net.Bluetooth.RadioMode.Discoverable
        '
        Using auther As New InTheHand.Net.Bluetooth.BluetoothWin32Authentication(handler)
            Console.WriteLine("Hit Return to complete")
            Console.ReadLine()
            Console.WriteLine("Complete")
        End Using
        radio.Mode = origRadioMode
    End Sub

    Private Win32AuthCallbackInitialBadPasscodeAndRetry_count As Integer
    Sub Win32AuthCallbackInitialBadPasscodeAndRetryHandler(ByVal sender As Object, ByVal e As InTheHand.Net.Bluetooth.BluetoothWin32AuthenticationEventArgs)
        Console.WriteLine("Authenticating {0} {1}", e.Device.DeviceAddress, e.Device.DeviceName)
        Console.WriteLine("  Attempt# {0}, Last error code {1}", e.AttemptNumber, e.PreviousNativeErrorCode)
        If e.AttemptNumber <> Win32AuthCallbackInitialBadPasscodeAndRetry_count Then
            Console.WriteLine("Bad AttemptNumber!!!")
        End If
        Dim rnd As New Random()
        Dim badPasscode As String = rnd.NextDouble().ToString()
        badPasscode = badPasscode.Substring(0, Math.Min(16, badPasscode.Length))
        '
        If Win32AuthCallbackInitialBadPasscodeAndRetry_count = 0 _
                OrElse Win32AuthCallbackInitialBadPasscodeAndRetry_count = 1 Then
            e.Pin = badPasscode
            e.CallbackWithResult = True
        ElseIf Win32AuthCallbackInitialBadPasscodeAndRetry_count = 2 _
                OrElse Win32AuthCallbackInitialBadPasscodeAndRetry_count = 3 Then
            e.Pin = "9876"
            e.CallbackWithResult = True
        ElseIf Win32AuthCallbackInitialBadPasscodeAndRetry_count = 4 Then
            e.CallbackWithResult = True ' Try to *wrongly* get another callback!
        Else
            Console.WriteLine("Unexpected callback #{0}", Win32AuthCallbackInitialBadPasscodeAndRetry_count)
        End If
        Console.WriteLine("Using '{0}'", IIf(e.Pin Is Nothing, "<null>", e.Pin))
        Win32AuthCallbackInitialBadPasscodeAndRetry_count += 1
    End Sub


    <TestFeature()> _
    Sub Win32AuthCallbackInduceFinalisationFault()
        Console.WriteLine("Authenticate a device two times")
        Console.WriteLine("Making PC discoverable")
        Dim radio As InTheHand.Net.Bluetooth.BluetoothRadio = InTheHand.Net.Bluetooth.BluetoothRadio.PrimaryRadio
        Dim origRadioMode As InTheHand.Net.Bluetooth.RadioMode = radio.Mode
        radio.Mode = InTheHand.Net.Bluetooth.RadioMode.Discoverable
        '
        Dim auther As Object
        auther = New InTheHand.Net.Bluetooth.BluetoothWin32Authentication(AddressOf Win32AuthCallbackTwoSeparateAuthenticationsHandler)
        Console.WriteLine("Hit Return to clear and GC")
        Console.ReadLine()
        auther = "foo"
        GC.Collect()
        Console.WriteLine("Hit Return to complete")
        Console.ReadLine()
        Console.WriteLine("Complete")
        radio.Mode = origRadioMode
    End Sub
End Class


Class TestFeatureAttribute
    Inherits Attribute

    Public Sub New()
    End Sub

End Class
